"""
This is a library to be able to use TC420 LED controller.

This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
from typing import Callable, Tuple, Union
import usb.core
from datetime import datetime, time
from struct import pack, unpack
from time import time as timestamp

from threading import Thread


class NoDeviceFoundError(ValueError):
    """ Exception when no TC420 device is found """


class TC420Packet:
    """
    TC420 USB communication packet
    """
    _magic = b"\x55\xaa"
    _command = b"\x00"

    def __init__(self, command=None, payload: Union[bytearray, bytes] = None) -> None:
        if command is None:
            command = self._command

        if payload is None:
            # Create 64 bytes long empty payload
            self.payload = bytearray(
                self._magic + command + b"\x00" * 59 + b"\x0d\x0a")
        else:
            self.payload = bytearray(payload)

        # 1st data position in payload
        self.pointer = 5

    def add_string(self, d: bytes, position: int = None) -> None:
        """
        Add string to the payload at the actual pointer or given position.
        If no position is givem the pointer is used for position and position will be incresed by data length.
        """
        if not d:
            return

        if position is None:  # If no absolute position is specified
            position = self.pointer  # We use our pointer as position
            self.pointer += len(d)  # Increase pointer
        elif position < 0:  # Support for negative position
            position = 64 + position

        assert position + len(d) <= 62

        self.payload[position:position + len(d)] = d

    def add_uchar(self, d: int, position: int = None) -> None:
        """ Add unsigned char to payload """
        self.add_string(pack("!B", d), position)

    def add_ushort(self, d: int, position: int = None) -> None:
        """ Add unsigned short to payload """
        self.add_string(pack("!H", d), position)

    def get_string(self, position=None, length=None):
        """
        Get string from given position or the pointer.
        If position is not given, the pointer is used,
        """
        p = position
        if position is None:
            p = self.pointer
        elif position < 0:
            p = 64 + position

        if length is None:
            length = 56 - position

        if position is None:
            self.pointer += length

        assert p + length <= 62

        return self.payload[p:p+length]

    def get_uchar(self, position: int = None) -> int:
        """ Get uchar from payload """
        return unpack("!B", self.get_string(position=position, length=1))[0]

    def get_ushort(self, position: int = None) -> int:
        """ Get ushort from payload """
        return unpack("!H", self.get_string(position=position, length=2))[0]

    def set_data_len(self, data_len=None):
        """
        Calculates length of data in payload
        This works only if you add data in order without position is specified
        """
        if data_len is None:
            data_len = self.pointer - 5
        assert data_len <= 56
        self.add_ushort(data_len, 3)

    def calc_checksum(self):
        """ Calculates simple checksum by adding all bytes together """
        checksum = 0
        for b in self.payload[:-3]:
            checksum = (checksum + b) & 0xff
        return checksum

    def set_checksum(self, checksum=None):
        """ Set packet checksum. If not specified, it is calculated by calc_checksum """
        if checksum is None:
            checksum = self.calc_checksum()
        self.add_uchar(checksum, -3)

    def __call__(self, update_data_len=True, update_checksum=True) -> bytes:
        """ Returns the final packet to send """
        if update_data_len:
            self.set_data_len()
        if update_checksum:
            self.set_checksum()
        return bytes(self.payload)

    @property
    def magic(self) -> bytes:
        """ Get the magic of the packet """
        return self.get_string(position=0, length=2)

    @property
    def data_len(self) -> int:
        """ Get data length """
        return self.get_ushort(position=3)

    @property
    def command(self) -> int:
        """ Get pacet command """
        return self.get_uchar(position=2)

    @property
    def data(self) -> bytes:
        """ Get packet data """
        return self.get_string(position=5, length=self.data_len)

    @property
    def checksum(self) -> int:
        """ Get packet checksum """
        return self.get_uchar(position=-3)


class TimeSyncPacket(TC420Packet):
    """
    Generates packet for time synchronization
    """
    _command = b"\x11"

    def __init__(self, date: datetime = None) -> None:
        if date is None:
            date = datetime.now()

        super().__init__()

        self.add_ushort(date.year)
        self.add_uchar(date.month)
        self.add_uchar(date.day)
        self.add_uchar(date.hour)
        self.add_uchar(date.minute)
        self.add_uchar(date.second)


class ModeInitPacket(TC420Packet):
    """
    Generates packet for mode initialization
    """
    _command = b"\x12"

    def __init__(self, step_count: int, name: str, bank: int = 0) -> None:
        assert step_count > 1, "Step count must be greater than 1!"
        assert 8 >= len(
            name) > 0, "Name length must be between 1 to 8 characters!"

        super().__init__()

        self.add_uchar(bank)
        self.add_uchar(step_count)
        self.add_string(name.encode('ascii', errors='replace'))


class ModeStepPacket(TC420Packet):
    """
    Generates packet for steps
    """
    _command = b"\x13"

    def __init__(self, step_data: Tuple[Union[datetime, time], int, int, int, int, int]) -> None:
        super().__init__()

        # Set jump flags
        jump_flags = 0
        step_data = list(step_data)
        for b, step in enumerate(step_data[1:]):
            # Negative step value means jump
            if step < 0:
                jump_flags |= 1 << b
                step_data[b + 1] = -step - 1

        self.add_uchar(step_data[0].hour)
        self.add_uchar(step_data[0].minute)
        self.add_uchar(min(100, abs(step_data[1])))  # CH1
        self.add_uchar(min(100, abs(step_data[2])))  # CH2
        self.add_uchar(min(100, abs(step_data[3])))  # CH3
        self.add_uchar(min(100, abs(step_data[4])))  # CH4
        self.add_uchar(min(100, abs(step_data[5])))  # CH5
        # The last byte represents the fade/jump flags
        self.add_uchar(jump_flags)


class ModeStepsStopPacket(TC420Packet):
    """
    Generate mode steps end packet
    """
    _command = b"\x01"


class ModeStopPacket(TC420Packet):
    """
    Generate mode steps end packet
    """
    _command = b"\x02"


class ModeClearAllPacket(TC420Packet):
    """
    Generate clear all modes packet
    """
    _command = b"\x03"


class PlayInitPacket(TC420Packet):
    """
    Initialize play mode packet
    """
    _command = b"\x15"

    def __init__(self, name: str) -> None:
        assert 8 >= len(
            name) > 0, "Name length must be between 1 to 8 characters!"

        super().__init__()

        self.add_string(name.encode('ascii', errors='replace'))
        # 0x7f in PLED.exe, but it seems it has no effect
        self.add_ushort(0x7f)


class PlaySetChannels(TC420Packet):
    """
    Set channels in play mode packet
    """
    _command = b"\x16"

    def __init__(self, step_data: Tuple[int, int, int, int, int]) -> None:
        super().__init__()

        self.add_uchar(0xf5)  # It is 0xf5 in PLED.exe, but it seems has no effect et all
        self.add_uchar(min(100, abs(step_data[0])))  # CH1
        self.add_uchar(min(100, abs(step_data[1])))  # CH2
        self.add_uchar(min(100, abs(step_data[2])))  # CH3
        self.add_uchar(min(100, abs(step_data[3])))  # CH4
        self.add_uchar(min(100, abs(step_data[4])))  # CH5
        self.add_uchar(0)  # Jump flags not working here


class TC420:
    VENDOR_ID = 0x0888
    PRODUCT_ID = 0x4000

    OK = b"\x00"

    timeout = 5000  # msec

    def __init__(self) -> None:
        # Initialize USB device
        self.dev = usb.core.find(idVendor=TC420.VENDOR_ID, idProduct=TC420.PRODUCT_ID)
        if self.dev is None:
            raise NoDeviceFoundError("TC420 device is not found!")

        # Detach kernel driver (hidraw0)
        if self.dev.is_kernel_driver_active(0):
            self.dev.detach_kernel_driver(0)

        cfg = self.dev[0]
        intf = cfg[(0, 0)]  # Interface - we have only one
        self.in_ep = intf[0]  # Input endpoint
        self.out_ep = intf[1]  # Output endpoint

        self._in_play = False
        self._play_step_data: Tuple[float, int, int, int, int, int] = None
        self._play_step_start: int = None

    def send(self, packet: TC420Packet, check=True) -> bool:
        """ Send a packet to device and wait for answer """
        self.out_ep.write(packet(), timeout=self.timeout)

        res_pkt = None
        if packet._command != ModeStepsStopPacket._command:  # No response for steps end (why?)
            res_pkt = TC420Packet(payload=self.in_ep.read(64, timeout=self.timeout))

        res = True
        if check and res_pkt:
            # The answer packets are not so well designed, they have no magic, no checksum...
            # It seems in case of error they just repeat the sent package from pos 2.
            res = res_pkt.data_len == 1 and res_pkt.data == self.OK

        return res

    def time_sync(self, date: datetime = None) -> bool:
        """
        Send date and time to the device
        :param date: The date and time you want to set on device, if not specified, the time of this computer is used
        """
        return self.send(TimeSyncPacket())

    def mode(self, name: str,
             steps: Tuple[Tuple[datetime, int, int, int, int, int]],
             bank: int = 0,
             step_ready_cb: Callable[[int], None] = None) -> bool:
        """
        Send mode name and steps to device
        :param name: The name of the mode
        :param steps: A tuple of tuple of the step data. One step has a datetime and 5 int in
                      range 0..100 for ch1..ch5. If value is negative, it is a jump (not fade) step.
        :param bank: The bank where to program shuld be placed
        :param step_ready_cb: Step ready callback, it is called after each step is sent.
        """
        assert len(steps) >= 2
        assert name
        assert bank < 64

        # Create mode init packet
        res = self.send(ModeInitPacket(
            name=name, step_count=len(steps), bank=bank))
        assert res, f"Mode '{name}' initialization failed!"
        # Send all steps one by one
        for i, step in enumerate(steps):
            res = self.send(ModeStepPacket(step_data=step))
            assert res, f"Setting step {i+1} in mode '{name}' has been failed!"
            if step_ready_cb is not None:
                step_ready_cb(i + 1)

        # End of steps (no response)
        self.send(ModeStepsStopPacket())

        return True

    def mode_stop(self) -> bool:
        """
        Send node stop to device. It is needed after all modes are sent.
        """
        self._in_play = False
        self._play_step_data = None
        self._play_step_start = None
        return self.send(ModeStopPacket())

    def mode_clear_all(self) -> bool:
        """
        Send clear all modes command to device
        """
        return self.send(ModeClearAllPacket())

    def play_step(self, step_data: Tuple[float, int, int, int, int, int]) -> bool:
        """
        Play step if we are in (fast)play mode
        """
        assert self._in_play, "Not in play mode!"
        self._play_step_data = step_data  # It is used in playing thread
        self._play_step_start = timestamp()
        return True

    def play(self, name: str,
             adapter: Callable[[int], Union[None, Tuple[int, int, int, int, int]]] = None,
             onchange_callback: Callable[[int, float, Tuple[int, int, int, int, int]], None] = None,
             wait=True) -> bool:
        """
        Switch to play mode, which is good to try different color level combinations

        When switch is done, the adapter is called (if specified), where you need to return with a
        tuple of next movement: (duration, CH1, CH2, CH3, CH4, CH5).

        :param name: The name of the fast play mode, it will be written to device LCD
        :param adapter: The value adapter function (callable)
        :param onchange_callback: If specified, it will be called with the actual index, elapsed time,
                                  and channel values
        """
        assert not self._in_play, "Already in play mode!"
        assert name, "Ypu must specify a name!"

        res = self.send(PlayInitPacket(name))
        assert res, "Switching to play mode was unsuccessfull!"

        self._play_step_data = None
        self._play_step_start = None
        self._in_play = True

        def playing_thread():
            """
            Continuously send actual step data for fast playing to keep it alive

            This actually works much better than in PLED.exe. That does not wait for device answer,
            just sends the same again and again like crazy. The device just sending back a lot of errors,
            and makes a lot of sync issue.
            """
            idx = 0
            last_channel_values = [0, 0, 0, 0, 0]
            cur_channel_values = [0, 0, 0, 0, 0]
            new_channel_values = [0, 0, 0, 0, 0]

            try:
                while self._in_play:
                    # Get new data from adapter if specified
                    if adapter is not None and self._play_step_data is None:
                        step_data = adapter(idx)
                        if step_data is None:
                            break
                        self.play_step(step_data)

                    if self._play_step_data is not None:
                        # The elapsed time
                        elapsed_time = timestamp() - self._play_step_start

                        # If timer is out
                        if elapsed_time >= self._play_step_data[0]:
                            last_channel_values = list(self._play_step_data[1:])
                            cur_channel_values = last_channel_values
                            idx += 1
                            self._play_step_data = None

                        else:
                            # Calculate new channel values
                            for c in range(5):
                                if self._play_step_data[c + 1] < 0:  # Immediate value
                                    new_channel_values[c] = abs(self._play_step_data[c + 1]) - 1
                                else:  # Linear calculation
                                    dcv = self._play_step_data[c + 1] - last_channel_values[c]
                                    new_channel_values[c] = round(last_channel_values[c] + dcv *
                                                                  (elapsed_time / self._play_step_data[0]))
                            # Apply new channel values
                            if new_channel_values != cur_channel_values:
                                cur_channel_values = new_channel_values

                        # Call onchange callback if values are changed
                        if onchange_callback:
                            onchange_callback(idx, elapsed_time, cur_channel_values)

                        # Send current channel values to device
                        self.send(PlaySetChannels(step_data=cur_channel_values))

            except KeyboardInterrupt:
                pass

        # Start playing thread
        t = Thread(target=playing_thread, daemon=True)
        t.start()

        # Block until thread is ended
        if wait:
            try:
                t.join()
            except KeyboardInterrupt as e:
                # Stop on keyboard interrupt
                self.stop()
                t.join()  # Wait for thread being stopped
                raise e

        return True

    def stop(self):
        assert self._in_play, "Not in play mode!"
        self._in_play = False


if __name__ == "__main__":
    print("This is a function library, it is not for execution.")



"""
TC420 CLI

This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
import sys
import os
from random import random, randint
from datetime import datetime
from typing import Callable, Tuple

import click
import usb.core
from .tc420 import TC420, NoDeviceFoundError


class CtxObj:
    """ This is the class used in context object to pass through all commands """
    dev: TC420 = None
    bank = -1
    mode_stop_neded = False


class Context(click.Context):
    """ This is just an interface for intellisense """
    obj: CtxObj


def check_result(res):
    """ Utility function to check if lib function result is good. Exit with error if not. """
    if res:
        print("OK.")
    else:
        print("ERROR!", file=sys.stderr)
        sys.exit(2)


def multiline_devider_replacer(f: Callable):
    """ Decorator to modify command line deviders in doc string according to OS """
    divider = '\\' if os.name != 'nt' else '^'
    f.__doc__ = f.__doc__.replace('{MULTILINE_DIVIDER}', divider)
    return f


@click.group(chain=True)
@click.pass_context
@multiline_devider_replacer
def cmd_group(ctx: Context):
    """
    TC420 LED Controller CLI Utility

    You can chain multiple commands (see below) together like this:

    \b
    tc420-cli {MULTILINE_DIVIDER}
        time-sync {MULTILINE_DIVIDER}
        mode -n test1 -s 00:00 0 0 0 0 0 -s 12:00 100 100 100 100 100 {MULTILINE_DIVIDER}
        mode -n test2 -s 05:00 10 10 30 0 0 -s 15:05 100 50 50 0 100

    If you need further help of a command, you can use --help option after the command name:

    \b
    tc420-cli mode --help
    """
    # Initialize context
    ctx.ensure_object(CtxObj)
    try:
        ctx.obj.dev = TC420()
    except usb.core.NoBackendError:
        print("No USB backend is available. Please install \"libusb\" for your platform!")
        sys.exit(1)
    except NoDeviceFoundError as e:
        print(str(e), file=sys.stderr)
        sys.exit(1)


@cmd_group.command()
@click.pass_context
def time_sync(ctx: Context):
    """
    Synchronize device time to this computer.
    """
    print("Syncing time...", end=" ", flush=True)
    check_result(ctx.obj.dev.time_sync())


@cmd_group.command(short_help="Create mode(s) (program).")
@click.pass_context
@click.option("--name", "-n", type=str, help="The name of the mode.", required=True,
              metavar="<MODE NAME>")
@click.option("--bank", "-b", type=click.IntRange(-1, 64),
              help="The program bank number mode will use. It is indexed from 0."
              "If it is not specified or it is -1 then automatic bank numbering is used.",
              metavar="<BANK NUMBER>", default=-1)
@click.option("--step", "-s", "steps", multiple=True, required=True,
              type=(
                  click.DateTime(formats=('%H:%M', '%H.%M', '%H%M')),
                  click.IntRange(-101, 100),
                  click.IntRange(-101, 100),
                  click.IntRange(-101, 100),
                  click.IntRange(-101, 100),
                  click.IntRange(-101, 100),
              ),
              metavar="<TIME> <CH1 %> <CH2 %> <CH3 %> <CH4 %> <CH5 %>",
              help="Step data: e.g.: 13:00 100 99 50 0 -70"
              "\n\nNegative channel values are \"jump\" (immediate) values, but because -0 is not "
              "interpretable, jump values are shifted by one. So -1 will be 0, -2 will be 1 ... and "
              "-101 will be 100.")
def mode(ctx: Context, name: str, bank: int, steps: Tuple[Tuple[datetime, int, int, int, int, int]]):
    """
    Create (or modify) mode(s) (program).

    If you want to create multiple modes, you may add multiple mode commands.

    You can specify program bank number, that way you can overwrite a specific mode.
    If no bank number is specified, the last mode's bank number plus one is used. If the 1st
    mode has no bank number, bank 0 is used. You can also mix auto and manual bank numbering.
    Please noted that if you leave gaps in banks, device will not skip them in LCD menu, there will be
    empty programs.
    """
    def step_ready_cb(step_num):
        print(f"{step_num}", end=" ", flush=True)

    # Automatic bank assignment
    if bank == -1:
        bank = ctx.obj.bank + 1
    ctx.obj.bank = bank

    print(f"Setting mode '{name}' in bank {bank}...", end=" ", flush=True)
    check_result(ctx.obj.dev.mode(name=name, bank=bank, steps=steps, step_ready_cb=step_ready_cb))
    ctx.obj.mode_stop_neded = True


@cmd_group.command()
@click.pass_context
def clear_all_modes(ctx: Context):
    """
    Clear all modes (programs) from the device.
    """
    print("Clearing all modes from device...", end=" ", flush=True)
    check_result(ctx.obj.dev.mode_clear_all())


@cmd_group.command(short_help='Fast Play a custom program without saving it to the device.')
@click.option("--name", "-n", type=str, help="The name of the play mode. "
              "(It will be written on the device screen)", required=True,
              metavar="<MODE NAME>")
@click.option("--steps", "-s", "steps", multiple=True, required=True,
              type=(
                  float,
                  click.IntRange(-101, 100),
                  click.IntRange(-101, 100),
                  click.IntRange(-101, 100),
                  click.IntRange(-101, 100),
                  click.IntRange(-101, 100),
              ),
              metavar="<DURATION sec> <CH1 %> <CH2 %> <CH3 %> <CH4 %> <CH5 %>",
              help="Step data: e.g.: 1.5 100 99 50 0 -70"
              "\n\nNegative channel values are \"jump\" (immediate) values, but because -0 is not "
              "interpretable, jump values are shifted by one. So -1 will be 0, -2 will be 1 ... and "
              "-101 will be 100.")
@click.pass_context
def play(ctx: Context, name: str, steps: Tuple[Tuple[float, int, int, int, int, int]]):
    """
    "Fast" Play a custom program without saving it to the device.
    In play mode you specify the time for fading into the new channel values instead of wall clock time.

    It is useful for e.g. Test your imagined program without waiting a whole day or testing effects, colors...

    You can place multiple --steps option to create a complete scene.
    """
    print(f"Initialize playing '{name}'...", end=" ", flush=True)

    ctx.obj.mode_stop_neded = True

    def adapter(step_idx: int):
        """ Callback to give the next step """
        try:
            try:
                return steps[step_idx]
            except IndexError:
                return None  # Stop iteration

        except KeyboardInterrupt:
            print("Cancelled.")
            return None  # Stop iteration

    def onchange_callback(idx: int, elapsed_time: float, channels: Tuple[int, int, int, int, int]):
        print(f"\rPlaying (CTRL+C to exit)... Idx: {idx:2}, time: {elapsed_time:4.1f}, "
              f"CH1: {channels[0]:3}, CH2: {channels[1]:3}, CH3: {channels[2]:3}, "
              f"CH4: {channels[3]:3}, CH5: {channels[4]:3}\r", end="", flush=True)

    try:
        ctx.obj.dev.play(name=name, adapter=adapter, onchange_callback=onchange_callback)
        print("")
    except KeyboardInterrupt:
        print("\rCancelled." + " " * 87)


@cmd_group.command()
@click.option("--channels", "-c", "channel_mask", default=(1, 1, 1, 1, 1),
              type=(
                  click.IntRange(0, 1),
                  click.IntRange(0, 1),
                  click.IntRange(0, 1),
                  click.IntRange(0, 1),
                  click.IntRange(0, 1),
),
    metavar="<<CH1> <CH2> <CH3> <CH4> <CH5>",
    help="Channel mask, 1 where you want the channel to be used, 0 where no use.")
@click.pass_context
def demo(ctx: Context, channel_mask: Tuple[int, int, int, int, int]):
    """
    "Fast" play a demo scene. You can stop it by pressing CTRL+C.

    This fades in-out channels randomly. Doesn't touch programs already on device.
    """
    print(f"Initialize playing 'Demo'...", end=" ", flush=True)

    ctx.obj.mode_stop_neded = True

    def rand_sleep(): return 0.0005 + (random() * 0.01)

    channels = [randint(0, 100) for _ in range(5)]  # Initial channel values
    directions = [randint(0, 1) for _ in range(5)]  # Channel directions
    sleeps = [rand_sleep() for _ in range(5)]  # Initial sleeps
    timers = [0.0] * 5
    started = False

    def adapter(_):
        """ Callback to give the next step """
        nonlocal started

        if not started:
            started = True
            print("OK.")

        changed = False

        while not changed:
            for i in range(5):
                timers[i] -= 0.0001   # Decrease timer

                if timers[i] <= 0:  # If a channel timer is elapsed
                    changed = True
                    # If a channel reached its end value
                    channels[i] += 1 if directions[i] == 1 else -1
                    if channel_mask[i] == 0:  # Channel masking support
                        channels[i] = 0
                    if channels[i] <= 0:
                        channels[i] = 0
                        directions[i] = 1  # Change direction
                        sleeps[i] = rand_sleep()  # Change speed
                    elif channels[i] >= 100:
                        channels[i] = 100
                        directions[i] = 0  # Change direction
                        sleeps[i] = rand_sleep()  # Change speed

                    timers[i] = sleeps[i]  # Restart timer

        return [0.0001, ] + channels

    def onchange(_, __, channels):
        print(f"\rPlaying (CTRL+C to exit)... CH1: {channels[0]:3}, CH2: {channels[1]:3}, CH3: {channels[2]:3}, "
              f"CH4: {channels[3]:3}, CH5: {channels[4]:3}\r", end="", flush=True)

    try:
        ctx.obj.dev.play(name="Demo", adapter=adapter, onchange_callback=onchange)
        print("")
    except KeyboardInterrupt:
        print("\rCancelled." + " " * 66)
        return None  # Stop iteration


@cmd_group.resultcallback()
@click.pass_context
def finalize(ctx: Context, _):
    """
    Finalize command chain
    """
    if ctx.obj.mode_stop_neded:
        # Close mode commands
        print("Finalizing modes...", end=" ", flush=True)
        check_result(ctx.obj.dev.mode_stop())


def main():
    # Start command group
    cmd_group()


if __name__ == '__main__':
    main()



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()
# We have list of requirements in the requirements.txt
REQUIREMENTS = (HERE / "requirements.txt").read_text().splitlines()

# This call to setup() does all the work
setup(
    name="tc420",
    version="0.2.1",
    description="TC420 LED Controller library and command line interface",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/wallneradam/tc420",
    author="Adam Wallner",
    author_email="adam.wallner@gmail.com",
    license="GPLv3+",
    packages=["tc420"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Environment :: Console",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Hardware :: Hardware Drivers",
        "Topic :: Home Automation",
    ],
    install_requires=REQUIREMENTS,
    entry_points={
        "console_scripts": [
            "tc420=tc420.__main__:main",
        ]
    },
)
